import os

import pytest

from mario.data_extractor import Configuration, HyperFile, DataExtractor
from mario.dataset_builder import DatasetBuilder, Format
from mario.dataset_specification import dataset_from_json, Constraint, dataset_from_manifest
from mario.metadata import metadata_from_json, metadata_from_manifest
from mario.query_builder import SubsetQueryBuilder


def test_integration_tdsx():
    dataset = dataset_from_json(os.path.join('test', 'dataset.json'))
    metadata = metadata_from_json(os.path.join('test', 'metadata.json'))
    configuration = Configuration(file_path=os.path.join('test', 'orders.hyper'))
    extractor = HyperFile(configuration=configuration, dataset_specification=dataset, metadata=metadata)
    builder = DatasetBuilder(dataset_specification=dataset, metadata=metadata, data=extractor)
    path = os.path.join('output', dataset.collection, dataset.name + '.tdsx')
    os.makedirs(os.path.join('output', dataset.collection), exist_ok=True)
    builder.build(file_path=path, output_format=Format.TABLEAU_PACKAGED_DATASOURCE)


def test_integration_csv():
    dataset = dataset_from_json(os.path.join('test', 'dataset.json'))
    metadata = metadata_from_json(os.path.join('test', 'metadata.json'))
    configuration = Configuration(file_path=os.path.join('test', 'orders.hyper'))
    extractor = HyperFile(configuration=configuration, dataset_specification=dataset, metadata=metadata)
    builder = DatasetBuilder(dataset_specification=dataset, metadata=metadata, data=extractor)
    path = os.path.join('output', dataset.collection, dataset.name + '.csv')
    os.makedirs(os.path.join('output', dataset.collection), exist_ok=True)
    builder.build(file_path=path, output_format=Format.CSV)


def test_integration_excel():
    dataset = dataset_from_manifest(os.path.join('test', 'manifest_superstore.json'))
    metadata = metadata_from_manifest(os.path.join('test', 'manifest_superstore.json'))
    configuration = Configuration(file_path=os.path.join('test', 'orders.hyper'))
    extractor = DataExtractor(configuration=configuration, dataset_specification=dataset, metadata=metadata)
    builder = DatasetBuilder(dataset_specification=dataset, metadata=metadata, data=extractor)
    path = os.path.join('output', 'test_integration_excel', 'data.xlsx')
    os.makedirs(os.path.join('output', 'test_integration_excel'), exist_ok=True)
    builder.build(file_path=path, output_format=Format.EXCEL_PIVOT, template_path='excel_template.xlsx')


@pytest.mark.skip
def test_sql_extraction():
    # Set up local test database, drivers and connection string to run this
    dataset = dataset_from_json(os.path.join('test', 'dataset.json'))
    metadata = metadata_from_json(os.path.join('test', 'metadata.json'))
    # Add constraints so we test parameter generation
    constraint = Constraint()
    constraint.item = 'Ship Mode'
    constraint.allowed_values = ['First Class']
    dataset.constraints.append(constraint)
    constraint = Constraint()
    constraint.item = 'City'
    constraint.allowed_values = ['Dallas']
    dataset.constraints.append(constraint)

    configuration = Configuration(
        connection_string=os.environ.get("CONNECTION_STRING"),
        schema='dbo',
        view='superstore',
        query_builder=SubsetQueryBuilder
    )
    extractor = DataExtractor(
        dataset_specification=dataset,
        metadata=metadata,
        configuration=configuration
    )
    extractor.validate_data()
    sql_path = os.path.join('output', 'test_sql_extraction', 'query.sql')
    csv_path = os.path.join('output', 'test_sql_extraction', 'data.csv')
    os.makedirs(os.path.join('output', 'test_sql_extraction'), exist_ok=True)
    extractor.save_data_as_csv(csv_path)
    extractor.save_query(sql_path)


@pytest.mark.skip
def test_sql_extraction_using_manifest():
    # Set up local test database, drivers and connection string to run this
    dataset = dataset_from_manifest(os.path.join('test', 'manifest_superstore.json'))
    metadata = metadata_from_manifest(os.path.join('test', 'manifest_superstore.json'))
    configuration = Configuration(
        connection_string=os.environ.get("CONNECTION_STRING"),
        schema='dbo',
        view='superstore',
        query_builder=SubsetQueryBuilder
    )
    extractor = DataExtractor(
        dataset_specification=dataset,
        metadata=metadata,
        configuration=configuration
    )
    extractor.validate_data()
    sql_path = os.path.join('output', 'test_sql_extraction_using_manifest', 'query.sql')
    csv_path = os.path.join('output', 'test_sql_extraction_using_manifest', 'data.csv')
    os.makedirs(os.path.join('output', 'test_sql_extraction_using_manifest'), exist_ok=True)
    extractor.save_data_as_csv(csv_path)
    extractor.save_query(sql_path)



import os

from mario.dataset_specification import dataset_from_json, dataset_from_manifest


def test_import_dataset():
    specification_file = os.path.join('test', 'dataset.json')
    specification = dataset_from_json(file_path=specification_file)

    assert specification.collection == 'superstore'
    assert len(specification.measures) == 4
    assert specification.get_property('description') is not None


def test_import_dataset_from_manifest():
    specification_file = os.path.join('test', 'manifest.json')
    specification = dataset_from_manifest(file_path=specification_file)

    assert specification.name == 'test@jisc.ac.uk'
    assert specification.collection == 'manifest_test'



import json
from typing import List
from mario.base import MarioBase


class Constraint:
    def __init__(self):
        self.item = None
        self.allowed_values = []


class DatasetSpecification(MarioBase):
    """
    Base class/interface for dataset specifications. Use an implementation specific to the
    format of specification used.
    """

    def __init__(self):
        super().__init__()
        self._name = ''
        self._collection: str = ''
        self._measures: List[str] = []
        self._dimensions: List[str] = []
        self._constraints: List[Constraint] = []

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def collection(self):
        return self._collection

    @collection.setter
    def collection(self, value):
        self._collection = value

    @property
    def enquiry(self):
        return ''

    @property
    def measures(self):
        return self._measures

    @measures.setter
    def measures(self, value):
        self._measures = value

    @property
    def dimensions(self):
        return self._dimensions

    @dimensions.setter
    def dimensions(self, value):
        self._dimensions = value

    @property
    def constraints(self):
        return self._constraints

    @constraints.setter
    def constraints(self, value):
        self._constraints = value

    @property
    def items(self):
        return self.dimensions + self.measures

    def save(self, file_path: str):
        json_representation = {
            "name": self.name,
            "collection": self.collection,
            "dimensions": self.dimensions,
            "measures": self.measures
        }
        with open(file_path, 'w') as file:
            json.dump(json_representation, file, default=vars)


def dataset_from_json(file_path: str = None) -> DatasetSpecification:
    with open(file_path, mode='r') as source_file:
        spec = json.load(source_file)

    dataset_specification = DatasetSpecification()
    dataset_specification.name = spec['name']
    dataset_specification.collection = spec['collection']
    dataset_specification.measures = spec['measures']
    dataset_specification.dimensions = spec['dimensions']
    dataset_specification.add_properties(source=spec, exclude=['name', 'collection', 'measures', 'dimensions'])

    return dataset_specification


def dataset_from_manifest(file_path: str = None):
    with open(file_path, mode='r') as source_file:
        spec = json.load(source_file)

    dataset_specification = DatasetSpecification()

    fields = []
    constraints = []
    for dimension in spec['items']:
        if 'allowedValues' in dimension and len(dimension['allowedValues']) > 0:
            constraint = Constraint()
            constraint.item = dimension['fieldName']
            constraint.allowed_values = dimension['allowedValues']
            constraints.append(constraint)
        else:
            fields.append(dimension['fieldName'])
    if 'onwardUseCategory' in spec:
        constraint = Constraint()
        constraint.item = 'Onward use category '+str(spec['onwardUseCategory'])
        constraint.allowed_values = [1]
        constraints.append(constraint)
    if 'years' in spec:
        constraint = Constraint()
        constraint.item = 'Academic year start'
        constraint.allowed_values = get_year_starts(spec['years'])
        constraints.append(constraint)

    dataset_specification.constraints = constraints
    dataset_specification.measures = [spec['measure']]
    dataset_specification.dimensions = fields
    dataset_specification.name = spec['client']
    dataset_specification.collection = spec['orderRef']
    dataset_specification.add_properties(source=spec, exclude=['name', 'orderRef', 'measure', 'items'])

    return dataset_specification


def get_year_starts(years):
    """
    Converts the array of academic year strings for a request into an array of year start integers
    :return: an array of integers for each year start
    """
    year_starts = []
    for year in years:
        year_start = int(year.split('/')[0])
        year_starts.append(year_start)
    return year_starts
